/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef REALTIMEBEATTRACKERINIT_HPP_
#define REALTIMEBEATTRACKERINIT_HPP_

typedef enum {STPM_METHOD, SELF_CORRELATION_METHOD} Beat_tracking_method;

class RealTimeBeatTrackerInit {
public:
    void set_beat_tracking_method(Beat_tracking_method beat_tracking_method) {
        beat_tracking_method_ = beat_tracking_method;
    };
    void set_sampling_freq(double sampling_freq) {
        sampling_freq_ = sampling_freq;
    };
    void SetSTFTParameter(size_t window_size,size_t shift_size) {
        window_size_   = window_size;//4096
        shift_size_    = shift_size;//512
    };
    void SetMelFilterParameter(size_t filter_num,size_t mel_f_start,size_t mel_f_end) {
        filter_num_    = filter_num;//64
        mel_f_start_   = mel_f_start;//1
        mel_f_end_     = mel_f_end;//63
    };
    // TODO : modify for parameterization of BPM
    void SetSTPMParameter(double cc_sizeBPM, double cc_maxBPM, double cc_minBPM) {
        cc_size_        = ceil (sampling_freq_/shift_size_*60/ cc_sizeBPM  );//1.01
        cc_shift_start_ = ceil(sampling_freq_/shift_size_*60/(cc_maxBPM-1));//0.51
        cc_shift_end_   = ceil (sampling_freq_/shift_size_*60/ cc_minBPM   );//1.01
    };
    void SetDebugParameter(bool voice_cancel_flag, bool debug_flag, std::string log_prefix_name) {
        voice_cancel_flag_ = voice_cancel_flag;//true
        debug_flag_ = debug_flag;//true
        log_prefix_name_ = log_prefix_name;//""
    };

    ////////////////////////////
    //RealTimeBeatTracker内で用いる変数
    ////////////////////////////
    Beat_tracking_method beat_tracking_method_;//メソッドの選択

    ////////////////////////////
    //BeatTrackerで用いる変数
    ////////////////////////////
    //入力スペクトルの仕様
    double sampling_freq_;//入力信号のサンプリングレート
    size_t window_size_,shift_size_;//FFT窓幅,SFTシフト幅

    //RealTimeBeatTracker パラメータ
    size_t filter_num_,mel_f_start_,mel_f_end_;//メルフィルタバンク次元,周波数（メル次元）スタート,エンド
    double cc_size_,cc_shift_start_,cc_shift_end_;//パターンマッチする幅, ずらす幅（スタート）,ずらす幅（おわり)

    //デバック用
    bool debug_flag_,voice_cancel_flag_;//デバックファイルを作るか
    std::string log_prefix_name_;//デバックファイルの接頭語
};

#endif /* REALTIMEBEATTRACKERINIT_HPP_ */
