/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef _RING_I_BUFFER_HPP_
#define _RING_I_BUFFER_HPP_

#include <vector>

//リングバッファーのインバース（最新から情報を抜く）
template<class T>
class RingIBuffer {
public:
    RingIBuffer(size_t size = 3000) {
        size_ = size;
        update_end_ = 0;
    }

    void set_size(size_t size) {
        size_ = size;
        vect.assign(size_, 0);
    }

    // references
    T& operator[](size_t i) {
        int number = update_end_-i;
        if (number < 0) {
            number += size_;
        }
        return vect[number];
    }

    const T& operator[](size_t i) const {
        int number = update_end_-i;
        if (number < 0) {
            number += size_;
        }
        return vect[number];
    }

    T& back() {
        return vect[update_end_];
    }

    const T& back() const {
        return vect[update_end_];
    }

    void push_back(T new_data) {
        ++update_end_;
        if (update_end_ >= size_)
            update_end_ -= size_;
        vect[update_end_] = new_data;
    }
    //template<class U>
    void init(T t) {
        vect.assign(size_, t);
    }

private:
    std::vector<T> vect;

    size_t size_;
    size_t update_end_;
};

#endif
