/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef _WINDOW_FUNCTION_H_
#define _WINDOW_FUNCTION_H_

#include <vector>
#include <cstddef>

class WindowFunction {
public:
    static const double pi;

    // window function type
    enum Type {
        WIN_RECTANGLE,
        WIN_HANNING,
        WIN_HAMMING,
        WIN_BARTLETT,
        WIN_BARTLETT_HANN,
        WIN_NUTTALL,
        WIN_BLACKMAN,
        WIN_BLACKMAN_HARRIS,
        WIN_BLACKMAN_NUTTALL,
        WIN_FLATTOP,
        WIN_AKAIKE,
        WIN_WELCH,
        WIN_ROSENFIELD
    };

    // constructor & destructor
    WindowFunction(size_t win_size, Type win_type = WIN_HANNING);
    ~WindowFunction()throw();

    size_t size()const;
    Type type()const;

    double operator[](size_t i)const;
    double value(size_t i)const;

private:
    Type win_type_;
    std::vector<double> data_;
};

#endif
